@@ -183,7 +183,7 @@ class ScanDataChannel:
             data_item.decrement_data_ref_count()
 
 
-class CameraDataChannel(scan_base.SynchronizedDataChannelInterface):
+class CameraDataChannel(camera_base.SynchronizedDataChannelInterface):
     title_base = _("MultiAcquire")
 
     def __init__(self, document_model: DocumentModel.DocumentModel, channel_name: str,
@@ -249,6 +249,7 @@ class CameraDataChannel(scan_base.SynchronizedDataChannelInterface):
             expected_ndim = int(is_sequence) + collection_dimension_count + datum_dimension_count
             if expected_ndim > len(data_shape):
                 collection_dimension_count = 0
+                data_calibrations = tuple()
         data_descriptor = DataAndMetadata.DataDescriptor(is_sequence, collection_dimension_count, datum_dimension_count)
         data_item.reserve_data(data_shape=data_shape, data_dtype=numpy.dtype(numpy.float32), data_descriptor=data_descriptor)
         dimensional_calibrations = scan_calibrations + data_calibrations
@@ -437,7 +438,7 @@ class SequenceBehavior:
             self.__last_shift += self.__multi_acquire_controller.active_spectrum_parameters[self.__current_parameters_index]['offset_x']
 
 
-SISequenceBehavior = collections.namedtuple('SISequenceBehavior', ['scan_behavior', 'scan_section_height',
+SISequenceBehavior = collections.namedtuple('SISequenceBehavior', ['scan_data_stream_functor', 'scan_section_height',
                                                                    'sequence_behavior', 'sequence_section_length'])
 
 class SISequenceAcquisitionHandler:
@@ -474,7 +475,7 @@ class SISequenceAcquisitionHandler:
                                                                      camera_data_channel=self.camera_data_channel,
                                                                      scan_frame_parameters=self.scan_frame_parameters,
                                                                      section_height=self.__si_sequence_behavior.scan_section_height,
-                                                                     scan_behavior=self.__si_sequence_behavior.scan_behavior)
+                                                                     scan_data_stream_functor=self.__si_sequence_behavior.scan_data_stream_functor)
             if combined_data is not None:
                 scan_xdata_list, _ = combined_data
                 data_channel_state = 'complete' if frame >= number_frames - 1 else 'partial'
@@ -793,6 +794,10 @@ class MultiAcquireController:
                         dark_data = numpy.sum(dark_data, axis=0)
                     else:
                         dark_data = numpy.mean(dark_data, axis=0)
+                    # since view mode will reset the processing flag and acquire_sequence uses the current frame
+                    # parameters, handle sum_project here instead.
+                    if self.__active_settings['processing'] == 'sum_project':
+                        dark_data = numpy.sum(dark_data, axis=0)
 
                     data_element['data'] -= dark_data
 
