@@ -20,6 +20,7 @@ from nion.swift import Facade
 from nion.swift.model import Graphics
 from nion.swift.model import Metadata
 from nion.swift.model import Schema
+from nion.swift.test import TestContext
 from nion.ui import TestUI
 from nion.utils import Geometry
 from nion.utils import Model
@@ -36,7 +37,7 @@ result = unittest.TextTestResult(sys.stdout, True, True)
 suite.run(result)
 """
 
-TIMEOUT = 20
+TIMEOUT = 30.0
 
 
 class ApplicationDataInMemory:
@@ -59,7 +60,7 @@ def make_scan_device(test_context: AcquisitionTestContext.test_context) -> Acqui
 
 
 def make_synchronized_device(test_context: AcquisitionTestContext.test_context) -> AcquisitionPanel.AcquisitionDeviceResult:
-    scan_context_description = AcquisitionPanel.SynchronizedScanDescription(str(), True, str(), Geometry.IntSize(6, 4), 0)
+    scan_context_description = AcquisitionPanel.SynchronizedScanDescription(str(), True, str(), Geometry.IntSize(6, 4), 0, 0, False)
     return AcquisitionPanel.build_synchronized_device_data_stream(test_context.scan_hardware_source, scan_context_description, test_context.camera_hardware_source)
 
 
@@ -103,9 +104,13 @@ def make_tableau_acquisition_method(adr: AcquisitionPanel.AcquisitionDeviceResul
 class TestCameraControlClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
         self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
-        self.exposure = 1.0
+        self.exposure = 0.04
+
+    def tearDown(self):
+        TestContext.end_leaks(self)
 
     def _acquire_one(self, document_controller: DocumentController.DocumentController, hardware_source: camera_base.CameraHardwareSource) -> None:
         hardware_source.start_playing()
@@ -123,7 +128,7 @@ class TestCameraControlClass(unittest.TestCase):
         document_controller.periodic()
 
     def __test_context(self, is_eels: bool=False):
-        return AcquisitionTestContext.test_context(is_eels=is_eels, camera_exposure=0.04)
+        return AcquisitionTestContext.test_context(is_eels=is_eels, camera_exposure=self.exposure)
 
     def __create_state_controller(self, acquisition_test_context: AcquisitionTestContext.AcquisitionTestContext, *,
                                   initialize: bool = True) -> CameraControlPanel.CameraControlStateController:
@@ -444,7 +449,7 @@ class TestCameraControlClass(unittest.TestCase):
             long_exposure = 0.5
             state_controller.handle_change_profile("Snap")
             state_controller.handle_binning_changed("4")
-            state_controller.handle_exposure_changed(str(int(long_exposure * 1000)))
+            state_controller.handle_exposure_changed(long_exposure)
             start = time.time()
             hardware_source.start_playing()
             try:
@@ -463,7 +468,7 @@ class TestCameraControlClass(unittest.TestCase):
             long_exposure = 0.5
             state_controller.handle_change_profile("Snap")
             state_controller.handle_binning_changed("4")
-            state_controller.handle_exposure_changed(str(int(long_exposure * 1000)))
+            state_controller.handle_exposure_changed(long_exposure)
             state_controller.handle_change_profile("Run")
             hardware_source.start_playing()
             try:
@@ -687,7 +692,7 @@ class TestCameraControlClass(unittest.TestCase):
             hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             display_item = document_model.get_display_item_for_data_item(document_model.data_items[0])
-            display_item.remove_graphic(display_item.graphics[0])
+            display_item.remove_graphic(display_item.graphics[0]).close()
             self._acquire_one(document_controller, hardware_source)
             self.assertEqual(display_item.graphics[0].bounds, hardware_source.data_channels[1].processor.bounds)
 
@@ -803,6 +808,26 @@ class TestCameraControlClass(unittest.TestCase):
             self.assertEqual(2, len(data_element["data"].shape))
             self.assertEqual(2, len(data_element["spatial_calibrations"]))
 
+    def test_ronchigram_calibrations(self):
+        with self.__test_context() as test_context:
+            document_controller = test_context.document_controller
+            document_model = test_context.document_model
+            hardware_source = test_context.camera_hardware_source
+            self._acquire_one(document_controller, hardware_source)
+            self.assertEqual("rad", document_model.data_items[0].dimensional_calibrations[0].units)
+            self.assertEqual("rad", document_model.data_items[0].dimensional_calibrations[1].units)
+
+    def test_eels_calibrations(self):
+        with self.__test_context(is_eels=True) as test_context:
+            document_controller = test_context.document_controller
+            document_model = test_context.document_model
+            hardware_source = test_context.camera_hardware_source
+            self._acquire_one(document_controller, hardware_source)
+            self.assertEqual("eV", document_model.data_items[0].dimensional_calibrations[1].units)
+            self.assertEqual("eV", document_model.data_items[1].dimensional_calibrations[0].units)
+            # note: it is an error to run view mode with "sum_project" enabled.
+            # "sum_project" is for sequence/SI only.
+
     def test_acquire_with_probe_position(self):
         # used to test out the code path, but no specific asserts
         with self.__test_context() as test_context:
@@ -1020,6 +1045,7 @@ class TestCameraControlClass(unittest.TestCase):
             mask = camera_base.Mask()
             graphic = Graphics.RectangleGraphic()
             mask.add_layer(graphic, 1.0)
+            graphic.close()
             frame_parameters.active_masks = [mask]
             self.assertEqual(frame_parameters.active_masks[0].to_dict(), mask.to_dict())
             self.assertEqual(frame_parameters["active_masks"][0].to_dict(), mask.to_dict())  # mask is already a dict
@@ -1103,11 +1129,20 @@ class TestCameraControlClass(unittest.TestCase):
                     h2 = AcquisitionPanel.SequenceAcquisitionMethodComponentHandler(c2, acquisition_preferences)
                     with contextlib.closing(h2):
                         adr = h.build_acquisition_device_data_stream()
-                        amr = h2.wrap_acquisition_device_data_stream(adr.data_stream, adr.device_map, adr.channel_names)
-                        ac._acquire_data_stream(amr.data_stream, amr.title_base, amr.channel_names, adr.drift_tracker)
-                        while ac.is_acquiring_model.value:
-                            document_controller.periodic()
-                        self.assertFalse(ac.is_error)
+                        try:
+                            amr = h2.wrap_acquisition_device_data_stream(adr.data_stream, adr.device_map, adr.channel_names)
+                            try:
+                                ac._acquire_data_stream(amr.data_stream, amr.title_base, amr.channel_names, adr.drift_tracker)
+                                start_time = time.time()
+                                while ac.is_acquiring_model.value:
+                                    document_controller.periodic()
+                                    time.sleep(0.1)
+                                    self.assertTrue(time.time() - start_time < TIMEOUT)
+                                self.assertFalse(ac.is_error)
+                            finally:
+                                amr.data_stream.remove_ref()
+                        finally:
+                            adr.data_stream.remove_ref()
             # only one data item will be create: the sequence. the view data item does not exist since acquiring
             # a sequence will use the special sequence acquisition of the camera device.
             self.assertEqual(1, len(document_controller.document_model.data_items))
@@ -1124,8 +1159,19 @@ class TestCameraControlClass(unittest.TestCase):
                                               amr.title_base,
                                               amr.channel_names,
                                               adr.drift_tracker)
+
+        start = time.time()
+        last_progress_time = time.time()
+        last_progress = progress_value_model.value
         while is_acquiring_model.value:
+            if time.time() - last_progress_time > TIMEOUT:
+                raise Exception(f"Timeout {TIMEOUT}s")
             document_controller.periodic()
+            progress = progress_value_model.value
+            if progress > last_progress:
+                last_progress = progress
+                last_progress_time = time.time()
+            time.sleep(0.05)
         self.assertFalse(acquisition_state.is_error)
         self.assertEqual(len(expected_dimensions), len(document_controller.document_model.data_items))
         for data_item, expected_dimension in zip(document_controller.document_model.data_items, expected_dimensions):
@@ -1173,8 +1219,33 @@ class TestCameraControlClass(unittest.TestCase):
                 with self.__test_context() as test_context:
                     document_controller = test_context.document_controller
                     adr = adr_fn(test_context)
-                    amr = amr_fn(adr)
-                    self.__test_acq(document_controller, adr, amr, expected_count)
+                    try:
+                        amr = amr_fn(adr)
+                        try:
+                            self.__test_acq(document_controller, adr, amr, expected_count)
+                        finally:
+                            amr.data_stream.remove_ref()
+                    finally:
+                        adr.data_stream.remove_ref()
+
+    def test_exposure_string(self):
+        t = (
+            (1.2E-0,  0,  "1.2", "s"),
+            (1.2E-0, -1,  "1.2", "s"),
+            (1.2E-0, -2, "1.20", "s"),
+            (1.2E-3, -3, "1.2", "ms"),
+            (1.2E-3, -4, "1.2", "ms"),
+            (1.2E-3, -5, "1.20", "ms"),
+            (1.2E-6, -6, "1.2", "us"),
+            (1.2E-6, -7, "1.2", "us"),
+            (1.2E-6, -8, "1.20", "us"),
+            (1.2E-9, -9, "1.2", "ns"),
+            (1.2E-9, -10, "1.2", "ns"),
+            (1.2E-9, -11, "1.20", "ns"),
+        )
+        for exposure, precision, exposure_s, units in t:
+            self.assertEqual(exposure_s, CameraControlPanel.make_exposure_str(exposure, precision))
+            self.assertEqual(units, CameraControlPanel.exposure_units[precision])
 
     def planned_test_custom_view_followed_by_ui_view_uses_ui_frame_parameters(self):
         pass
