@@ -8,17 +8,17 @@ import unittest
 import uuid
 
 from nion.data import DataAndMetadata
-from nion.instrumentation import HardwareSource
 from nion.swift import Application
 from nion.swift import Facade
 from nion.swift.model import ApplicationData
 from nion.swift.model import Metadata
+from nion.swift.test import TestContext
 from nion.ui import TestUI
 from nion.utils import Geometry
 from nion.instrumentation import Acquisition
+from nion.instrumentation import camera_base
 from nion.instrumentation import DataChannel
 from nion.instrumentation import DriftTracker
-from nion.instrumentation import camera_base
 from nion.instrumentation import scan_base
 from nion.instrumentation.test import AcquisitionTestContext
 from nionswift_plugin.nion_instrumentation_ui import ScanAcquisition
@@ -32,11 +32,49 @@ result = unittest.TextTestResult(sys.stdout, True, True)
 suite.run(result)
 """
 
+
+class ScanMover(Acquisition.ContainerDataStream):
+    def __init__(self, scan_hardware_source: scan_base.ScanHardwareSource, drift: Geometry.FloatSize, data_stream: Acquisition.DataStream) -> None:
+        super().__init__(data_stream)
+        self.__scan_hardware_source = scan_hardware_source
+        self.__drift = drift
+        self.__skip = True  # skip adjusting the stage the first time through
+
+    def _prepare_stream(self, stream_args: Acquisition.DataStreamArgs, **kwargs: typing.Any) -> None:
+        # adjust the stage for testing.
+        if not self.__skip:
+            stem_controller = self.__scan_hardware_source.stem_controller
+            # print()
+            # print(f"move delta {self.__drift}")
+            stem_controller.SetValDeltaAndConfirm("CSH.x", self.__drift.width, 1.0, 1000)
+            stem_controller.SetValDeltaAndConfirm("CSH.y", self.__drift.height, 1.0, 1000)
+        self.__skip = False
+        # call this last so that we move the stage before measuring drift.
+        super()._prepare_stream(stream_args, **kwargs)
+
+
+class ScanMoverFunctor(Acquisition.DataStreamFunctor):
+    def __init__(self, scan_hardware_source: scan_base.ScanHardwareSource, drift: Geometry.FloatSize, functor: Acquisition.DataStreamFunctor) -> None:
+        self.__scan_hardware_source = scan_hardware_source
+        self.__drift = drift
+        self.__functor = functor
+
+    def apply(self, data_stream: Acquisition.DataStream) -> Acquisition.DataStream:
+        # the functor passed to this functor will typically be the drift corrector
+        # ensure that the stage is moved first, then the drift corrector measures drift,
+        # then the scan section is prepared (using the measured drift).
+        return ScanMover(self.__scan_hardware_source, self.__drift, self.__functor.apply(data_stream))
+
+
 class TestSynchronizedAcquisitionClass(unittest.TestCase):
 
     def setUp(self):
+        TestContext.begin_leaks()
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
 
+    def tearDown(self):
+        TestContext.end_leaks(self)
+
     def __test_context(self, *, is_eels: bool = False) -> AcquisitionTestContext.AcquisitionTestContext:
         return AcquisitionTestContext.test_context(is_eels=is_eels)
 
@@ -108,23 +146,32 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             camera_frame_parameters.processing = "sum_project"
             camera_data_channel = None
 
-            class TestAbortBehavior(scan_base.SynchronizedScanBehaviorInterface):
-                def __init__(self) -> None:
+            class TestAbortDataStream(Acquisition.ContainerDataStream):
+                def __init__(self, scan_hardware_source: scan_base.ScanHardwareSource, data_stream: Acquisition.DataStream) -> None:
+                    super().__init__(data_stream)
+                    self.__scan_hardware_source = scan_hardware_source
                     self.__i = 0
 
-                def prepare_section(self, **kwargs) -> None:
+                def _prepare_stream(self, stream_args: Acquisition.DataStreamArgs, **kwargs: typing.Any) -> None:
                     self.__i += 1
                     if self.__i == 2:
-                        scan_hardware_source.grab_synchronized_abort()
+                        self.__scan_hardware_source.grab_synchronized_abort()
+                    super()._prepare_stream(stream_args, **kwargs)
+
+            class TestAbortBehaviorFunctor(Acquisition.DataStreamFunctor):
+                def __init__(self, scan_hardware_source: scan_base.ScanHardwareSource) -> None:
+                    self.__scan_hardware_source = scan_hardware_source
+
+                def apply(self, data_stream: Acquisition.DataStream) -> Acquisition.DataStream:
+                    return TestAbortDataStream(self.__scan_hardware_source, data_stream)
 
-            abort_behavior = TestAbortBehavior()
             scans_and_spectrum_images = scan_hardware_source.grab_synchronized(
                 scan_frame_parameters=scan_frame_parameters,
                 camera=camera_hardware_source,
                 camera_frame_parameters=camera_frame_parameters,
                 camera_data_channel=camera_data_channel,
                 section_height=2,
-                scan_behavior=abort_behavior)
+                scan_data_stream_functor=TestAbortBehaviorFunctor(scan_hardware_source))
             self.assertIsNone(scans_and_spectrum_images)
             # check the acquisition state
             self.assertFalse(camera_hardware_source.camera._is_acquire_synchronized_running)
@@ -151,6 +198,28 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             # self.assertEqual((4, 4, 512), spectrum_images[0].data_shape)  # assumes accumulate for now
             self.assertEqual((3, 4, 4, 512), spectrum_images[0].data_shape)
 
+    def test_grab_synchronized_basic_eels_followed_by_view(self):
+        # perform a synchronized acquisition followed by a record. tests that the record frame parameters are restored
+        # after a synchronized acquisition.
+        with self.__test_context(is_eels=True) as test_context:
+            document_controller = test_context.document_controller
+            scan_hardware_source = test_context.scan_hardware_source
+            camera_hardware_source = test_context.camera_hardware_source
+            self._acquire_one(document_controller, test_context.camera_hardware_source)
+            self.assertEqual("eV", document_controller.document_model.data_items[0].dimensional_calibrations[-1].units)
+            scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
+            scan_frame_parameters.scan_id = uuid.uuid4()
+            scan_frame_parameters.size = Geometry.IntSize(4, 4)
+            camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
+            camera_frame_parameters.processing = "sum_project"
+            camera_data_channel = None
+            old_frame_parameters_d = camera_hardware_source.get_current_frame_parameters().as_dict()
+            scan_hardware_source.grab_synchronized(scan_frame_parameters=scan_frame_parameters, camera=camera_hardware_source, camera_frame_parameters=camera_frame_parameters, camera_data_channel=camera_data_channel)
+            self.assertEqual(old_frame_parameters_d, camera_hardware_source.get_current_frame_parameters().as_dict())
+            document_controller.periodic()
+            self._acquire_one(document_controller, test_context.camera_hardware_source)
+            self.assertEqual("eV", document_controller.document_model.data_items[0].dimensional_calibrations[-1].units)
+
     def test_grab_synchronized_basic_eels_followed_by_record(self):
         # perform a synchronized acquisition followed by a record. tests that the record frame parameters are restored
         # after a synchronized acquisition.
@@ -453,10 +522,11 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             scan_hardware_source = test_context.scan_hardware_source
+            test_context.instrument.sample_index = 2  # use CTS sample, custom position chosen using view mode in Swift
             camera_hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, scan_hardware_source)
             scan_hardware_source.drift_channel_id = scan_hardware_source.data_channels[0].channel_id
-            scan_hardware_source.drift_region = Geometry.FloatRect.from_tlhw(0.25, 0.25, 0.5, 0.5)
+            scan_hardware_source.drift_region = Geometry.FloatRect.from_center_and_size(Geometry.FloatPoint(0.6554, 0.2932), Geometry.FloatSize(0.15, 0.15))
             document_controller.periodic()
             scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
             scan_frame_parameters.scan_id = uuid.uuid4()
@@ -464,13 +534,19 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
             camera_frame_parameters.processing = "sum_project"
             camera_data_channel = None
-            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(scan_hardware_source, scan_frame_parameters)
+            drift = Geometry.FloatSize(0.0, 1.5e-9)
+            # we cannot use drift prediction since we are running a test using the scan mover, which is not continuous.
+            # this is still measuring drift, just not using timing to predict its future location.
+            drift_correction_functor = DriftTracker.DriftCorrectionDataStreamFunctor(scan_hardware_source, scan_frame_parameters, scan_hardware_source.drift_tracker, 0, use_prediction=False)
+            scan_mover_functor = ScanMoverFunctor(scan_hardware_source, drift, drift_correction_functor)
             scans, spectrum_images = scan_hardware_source.grab_synchronized(scan_frame_parameters=scan_frame_parameters,
                                                                             camera=camera_hardware_source,
                                                                             camera_frame_parameters=camera_frame_parameters,
                                                                             camera_data_channel=camera_data_channel,
                                                                             section_height=2,
-                                                                            scan_behavior=drift_correction_behavior)
+                                                                            scan_data_stream_functor=scan_mover_functor)
+            self.assertEqual(3, scan_hardware_source.drift_tracker.measurement_count)
+            self.assertTrue(1.4 < numpy.mean(scan_hardware_source.drift_tracker.drift_data_frame[2]) < 1.6)
 
     def test_grab_synchronized_basic_eels_with_drift_correction_leaves_graphic_during_scan(self):
         with self.__test_context(is_eels=True) as test_context:
@@ -490,14 +566,14 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
             camera_frame_parameters.processing = "sum_project"
             camera_data_channel = None
-            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(scan_hardware_source, scan_frame_parameters)
+            drift_correction_functor = DriftTracker.DriftCorrectionDataStreamFunctor(scan_hardware_source, scan_frame_parameters, scan_hardware_source.drift_tracker, 0)
             def do_grab():
                 scans, spectrum_images = scan_hardware_source.grab_synchronized(scan_frame_parameters=scan_frame_parameters,
                                                                                 camera=camera_hardware_source,
                                                                                 camera_frame_parameters=camera_frame_parameters,
                                                                                 camera_data_channel=camera_data_channel,
                                                                                 section_height=2,
-                                                                                scan_behavior=drift_correction_behavior)
+                                                                                scan_data_stream_functor=drift_correction_functor)
             # run the grab synchronized in a thread so that periodic can be called so that
             # the graphics get updated in ui thread.
             t = threading.Thread(target=do_grab)
@@ -531,14 +607,14 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
             camera_frame_parameters.processing = "sum_project"
             camera_data_channel = None
-            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(scan_hardware_source, scan_frame_parameters)
+            drift_correction_functor = DriftTracker.DriftCorrectionDataStreamFunctor(scan_hardware_source, scan_frame_parameters, scan_hardware_source.drift_tracker, 0)
             def do_grab():
                 scans, spectrum_images = scan_hardware_source.grab_synchronized(scan_frame_parameters=scan_frame_parameters,
                                                                                 camera=camera_hardware_source,
                                                                                 camera_frame_parameters=camera_frame_parameters,
                                                                                 camera_data_channel=camera_data_channel,
                                                                                 section_height=2,
-                                                                                scan_behavior=drift_correction_behavior)
+                                                                                scan_data_stream_functor=drift_correction_functor)
             # run the grab synchronized in a thread so that periodic can be called so that
             # the graphics get updated in ui thread.
             t = threading.Thread(target=do_grab)
@@ -552,31 +628,33 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             self.assertEqual(drift_graphic, display_item.graphics[-1])
 
     def test_drift_corrector(self):
+        # for this test, drift should be in a constant direction
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             scan_hardware_source = test_context.scan_hardware_source
+            test_context.instrument.sample_index = 2  # use CTS sample, custom position chosen using view mode in Swift
             drift_tracker = scan_hardware_source.drift_tracker
             self._acquire_one(document_controller, scan_hardware_source)
             scan_hardware_source.drift_channel_id = scan_hardware_source.data_channels[0].channel_id
-            scan_hardware_source.drift_region = Geometry.FloatRect.from_tlhw(0.25, 0.25, 0.5, 0.5)
+            scan_hardware_source.drift_region = Geometry.FloatRect.from_center_and_size(Geometry.FloatPoint(0.6554, 0.2932), Geometry.FloatSize(0.15, 0.15))
             document_controller.periodic()
             scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
-            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(scan_hardware_source, scan_frame_parameters)
+            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(drift_tracker, scan_hardware_source, scan_frame_parameters, 0)
             self.assertEqual(0.0, drift_tracker.last_delta_nm.width)
             self.assertEqual(0.0, drift_tracker.last_delta_nm.height)
             drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
             self.assertLess(dist_nm, 0.1)
-            stem_controller = HardwareSource.HardwareSourceManager().get_instrument_by_id("usim_stem_controller")
-            stem_controller.SetValDeltaAndConfirm("CSH.x", 2e-9, 1.0, 1000)
+            stem_controller = test_context.instrument
+            stem_controller.SetValDeltaAndConfirm("CSH.x", 1e-9, 1.0, 1000)
             drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
-            self.assertTrue(1.9 < dist_nm < 2.1)
-            self.assertTrue(1.9 < abs(last_delta_nm.width) < 2.1)
+            self.assertTrue(0.9 < dist_nm < 1.1)
+            self.assertTrue(0.9 < abs(last_delta_nm.width) < 1.1)
             self.assertTrue(abs(last_delta_nm.height) < 0.1)
-            stem_controller.SetValDeltaAndConfirm("CSH.x", -2e-9, 1.0, 1000)
+            stem_controller.SetValDeltaAndConfirm("CSH.x", 2e-9, 1.0, 1000)
             drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
@@ -585,18 +663,20 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             self.assertTrue(abs(last_delta_nm.height) < 0.1)
 
     def test_drift_corrector_with_drift_sub_area_rotation(self):
+        # for this test, drift should be in a constant direction
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             scan_hardware_source = test_context.scan_hardware_source
+            test_context.instrument.sample_index = 2  # use CTS sample, custom position chosen using view mode in Swift
             drift_tracker = scan_hardware_source.drift_tracker
             self._acquire_one(document_controller, scan_hardware_source)
             scan_hardware_source.drift_channel_id = scan_hardware_source.data_channels[0].channel_id
-            scan_hardware_source.drift_region = Geometry.FloatRect.from_tlhw(0.25, 0.25, 0.5, 0.5)
-            rotation = math.radians(30)
+            scan_hardware_source.drift_region = Geometry.FloatRect.from_center_and_size(Geometry.FloatPoint(0.6554, 0.2932), Geometry.FloatSize(0.15, 0.15))
+            rotation = math.radians(30)  # note: this is only rotating the drift region, not the overall sample
             scan_hardware_source.drift_rotation = rotation
             document_controller.periodic()
             scan_frame_parameters = scan_hardware_source.get_current_frame_parameters()
-            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(scan_hardware_source, scan_frame_parameters)
+            drift_correction_behavior = DriftTracker.DriftCorrectionBehavior(drift_tracker, scan_hardware_source, scan_frame_parameters, 0)
             self.assertEqual(0.0, drift_tracker.last_delta_nm.width)
             self.assertEqual(0.0, drift_tracker.last_delta_nm.height)
             # offset will be rotated into the context reference frame
@@ -604,15 +684,15 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
             self.assertLess(dist_nm, 0.1)
-            stem_controller = HardwareSource.HardwareSourceManager().get_instrument_by_id("usim_stem_controller")
-            stem_controller.SetValDeltaAndConfirm("CSH.x", 2e-9, 1.0, 1000)
+            stem_controller = test_context.instrument
+            stem_controller.SetValDeltaAndConfirm("CSH.x", 1e-9, 1.0, 1000)
             drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
-            self.assertTrue(1.9 < dist_nm < 2.1)
-            self.assertTrue(1.9 < abs(last_delta_nm.width) < 2.1)
+            self.assertTrue(0.9 < dist_nm < 1.1)
+            self.assertTrue(0.9 < abs(last_delta_nm.width) < 1.1)
             self.assertTrue(abs(last_delta_nm.height) < 0.1)
-            stem_controller.SetValDeltaAndConfirm("CSH.x", -2e-9, 1.0, 1000)
+            stem_controller.SetValDeltaAndConfirm("CSH.x", 2e-9, 1.0, 1000)
             drift_correction_behavior.prepare_section(utc_time=drift_tracker._last_entry_utc_time)
             last_delta_nm = drift_tracker.last_delta_nm
             dist_nm = math.sqrt(pow(last_delta_nm.width, 2) + pow(last_delta_nm.height, 2))
@@ -864,6 +944,26 @@ class TestSynchronizedAcquisitionClass(unittest.TestCase):
             self.assertIsNotNone(current_frame_parameters.subscan_fractional_size)
             self.assertIsNotNone(current_frame_parameters.subscan_pixel_size)
 
+    def test_grab_sequence_basic_eels_followed_by_view(self):
+        with self.__test_context(is_eels=True) as test_context:
+            document_controller = test_context.document_controller
+            camera_hardware_source = test_context.camera_hardware_source
+            # establish first acquisition
+            self._acquire_one(document_controller, test_context.camera_hardware_source)
+            self.assertEqual("eV", document_controller.document_model.data_items[0].dimensional_calibrations[-1].units)
+            # perform a sequence acquisition. ensure get camera frame parameters are restored.
+            old_frame_parameters_d = camera_hardware_source.get_current_frame_parameters().as_dict()
+            camera_frame_parameters = camera_hardware_source.get_current_frame_parameters()
+            camera_frame_parameters.processing = "sum_project"
+            data_stream = camera_base.make_sequence_data_stream(camera_hardware_source, camera_frame_parameters, 4)
+            maker = Acquisition.FramedDataStream(data_stream)
+            with maker.ref():
+                Acquisition.acquire(maker)
+            self.assertEqual(old_frame_parameters_d, camera_hardware_source.get_current_frame_parameters().as_dict())
+            # another acquisition. verify it is calibrated.
+            self._acquire_one(document_controller, test_context.camera_hardware_source)
+            self.assertEqual("eV", document_controller.document_model.data_items[0].dimensional_calibrations[-1].units)
+
     # TODO: check for counts per electron
 
 
